@@ -243,28 +243,39 @@ SKIP: {
     # test reformed UNIVERSAL::VERSION
     diag "Replacement UNIVERSAL::VERSION tests" if $Verbose;
     
-    # we know this file is here since we require it ourselves
-    $version = $Test::More::VERSION;
-    eval "use Test::More $version";
-    unlike($@, qr/Test::More version $version/,
-	    'Replacement eval works with exact version');
-    
-    # this should fail even with old UNIVERSAL::VERSION
-    $version = $Test::More::VERSION+0.01;
-    eval "use Test::More $version";
-    like($@, qr/Test::More version $version/,
-	    'Replacement eval works with incremented version');
-    
-    $version =~ s/\.0$//; #convert to string and remove trailing '.0'
-    chop($version);	# shorten by 1 digit, should still succeed
-    eval "use Test::More $version";
-    unlike($@, qr/Test::More version $version/,
-	    'Replacement eval works with single digit');
-    
-    $version += 0.1; # this would fail with old UNIVERSAL::VERSION
-    eval "use Test::More $version";
-    like($@, qr/Test::More version $version/,
-	    'Replacement eval works with incremented digit');
+    {
+	open F, ">aaa.pm" or die "Cannot open aaa.pm: $!\n";
+	print F "package aaa;\n\$aaa::VERSION=0.58;\n1;\n";
+	close F;
+
+	$version = 0.58; $version = sprintf("%.3f",$version);
+	eval "use lib '.'; use aaa $version";
+	unlike($@, qr/aaa version $version/,
+		'Replacement eval works with exact version');
+	
+	# test as class method
+	$new_version = "aaa"->VERSION;
+	cmp_ok($new_version,'eq',$version, "Called as class method");
+
+	# this should fail even with old UNIVERSAL::VERSION
+	$version += 0.01; $version = sprintf("%.3f",$version);
+	eval "use lib '.'; use aaa $version";
+	like($@, qr/aaa version $version/,
+		'Replacement eval works with incremented version');
+	
+	$version =~ s/0+$//; #convert to string and remove trailing 0's
+	chop($version);	# shorten by 1 digit, should still succeed
+	eval "use lib '.'; use aaa $version";
+	unlike($@, qr/aaa version $version/,
+		'Replacement eval works with single digit');
+	
+	# this would fail with old UNIVERSAL::VERSION
+	$version += 0.1; $version = sprintf("%.3f",$version);
+	eval "use lib '.'; use aaa $version";
+	like($@, qr/aaa version $version/,
+		'Replacement eval works with incremented digit');
+	unlink 'aaa.pm';
+    }
 
     { # dummy up some variously broken modules for testing
 	open F, ">xxx.pm" or die "Cannot open xxx.pm: $!\n";
@@ -281,9 +292,10 @@ SKIP: {
 	eval "use lib '.'; use xxx 3;";
 	like ($@, qr/$error_regex/,
 	    'Replacement handles modules without package or VERSION'); 
-	eval "use lib '.'; use xxx; print xxx->VERSION";
+	eval "use lib '.'; use xxx; $version = xxx->VERSION";
 	unlike ($@, qr/$error_regex/,
 	    'Replacement handles modules without package or VERSION'); 
+	ok (defined($version), "Called as class method");
 	unlink 'xxx.pm';
     }
     
@@ -314,8 +326,8 @@ SKIP: {
     }
 
 SKIP: 	{
-	skip 'Cannot test bare v-strings with Perl < 5.8.1', 4
-		if $] < 5.008_001; 
+	skip 'Cannot test bare v-strings with Perl < 5.6.0', 4
+		if $] < 5.006_000; 
 	diag "Tests with v-strings" if $Verbose;
 	$version = $CLASS->new(1.2.3);
 	ok("$version" eq "v1.2.3", '"$version" eq 1.2.3');
@@ -349,11 +361,28 @@ SKIP: 	{
     $version = $CLASS->new(" 1.7");
     ok($version->numify eq "1.700", "leading space ignored");
 
-SKIP: {
-	$DB::single = 1;
-	skip "version require'd instead of use'd, cannot test qv", 4
-	    if defined $no_qv;
+    # RT 19517 - deal with undef and 'undef' initialization
+    ok($version ne 'undef', "Undef version comparison #1");
+    ok($version ne undef, "Undef version comparison #2");
+    $version = $CLASS->new('undef');
+    unlike($warning, qr/^Version string 'undef' contains invalid data/,
+	"Version string 'undef'");
 
+    $version = $CLASS->new(undef);
+    like($warning, qr/^Use of uninitialized value/,
+	"Version string 'undef'");
+    ok($version eq 'undef', "Undef version comparison #3");
+    ok($version eq undef, "Undef version comparison #4");
+    eval "\$version = \$CLASS->new()"; # no parameter at all
+    unlike($@, qr/^Bizarre copy of CODE/, "No initializer at all");
+    ok($version eq 'undef', "Undef version comparison #5");
+    ok($version eq undef, "Undef version comparison #6");
+
+    $version = $CLASS->new(0.000001);
+    unlike($warning, qr/^Version string '1e-06' contains invalid data/,
+    	"Very small version objects");
+
+SKIP: {
 	# dummy up a legal module for testing RT#19017
 	open F, ">www.pm" or die "Cannot open www.pm: $!\n";
 	print F <<"EOF";
@@ -371,16 +400,230 @@ EOF
 	    "Comparing vs. version with no decimal"); 
 	eval "use lib '.'; use www 1.;";
 	like ($@, qr/^www version 1.000 \(v1.0.0\) required/,
-	    "Comparing "); 
+	    "Comparing vs. version with decimal only"); 
 
-	skip 'Cannot "use" extended versions with Perl < 5.6.2', 1
-	    if $] < 5.006_002;
+	if ( $] < 5.006_002 ) {
+	    unlink 'www.pm';
+	    skip 'Cannot "use" extended versions with Perl < 5.6.2', 3; 
+	}
 	eval "use lib '.'; use www 0.0.8;";
 	like ($@, qr/^www version 0.000008 \(v0.0.8\) required/,
 	    "Make sure very small versions don't freak"); 
 
+	eval "use lib '.'; use www 0.0.4;";
+	unlike($@, qr/^www version 0.000004 \(v0.0.4\) required/,
+	    'Succeed - required == VERSION');
+	cmp_ok ( "www"->VERSION, 'eq', '0.000004', 'No undef warnings' );
+
 	unlink 'www.pm';
     }
+
+    open F, ">vvv.pm" or die "Cannot open vvv.pm: $!\n";
+    print F <<"EOF";
+package vvv;
+use base qw(version);
+1;
+EOF
+    close F;
+    # need to eliminate any other qv()'s
+    undef *main::qv;
+    ok(!defined(&{"main\::qv"}), "make sure we cleared qv() properly");
+    eval "use lib '.'; use vvv;";
+    ok(defined(&{"main\::qv"}), "make sure we exported qv() properly");
+    isa_ok( qv(1.2), "vvv");
+    unlink 'vvv.pm';
+
+SKIP: {
+	# test locale handling
+	my $ver = 1.23;  # has to be floating point number
+	my $loc;
+	while (<DATA>) {
+	    chomp;
+	    $loc = POSIX::setlocale( &POSIX::LC_ALL, $_);
+	    last if POSIX::localeconv()->{decimal_point} eq ',';
+	}
+	skip 'Cannot test locale handling without a comma locale', 4
+	    unless ( $loc and ($ver eq '1,23') );
+
+	diag ("Testing locale handling with $loc") if $Verbose;
+
+	my $v = $CLASS->new($ver);
+	unlike($warning,qr/Version string '1,23' contains invalid data/,
+	    "Process locale-dependent floating point");
+	is ($v, "1.230", "Locale doesn't apply to version objects");
+	ok ($v == $ver, "Comparison to locale floating point");
+    }
 }
 
 1;
+
+__DATA__
+af_ZA
+af_ZA.utf8
+an_ES
+an_ES.utf8
+az_AZ.utf8
+be_BY
+be_BY.utf8
+bg_BG
+bg_BG.utf8
+br_FR
+br_FR@euro
+br_FR.utf8
+bs_BA
+bs_BA.utf8
+ca_ES
+ca_ES@euro
+ca_ES.utf8
+cs_CZ
+cs_CZ.utf8
+da_DK
+da_DK.utf8
+de_AT
+de_AT@euro
+de_AT.utf8
+de_BE
+de_BE@euro
+de_BE.utf8
+de_DE
+de_DE@euro
+de_DE.utf8
+de_LU
+de_LU@euro
+de_LU.utf8
+el_GR
+el_GR.utf8
+en_DK
+en_DK.utf8
+es_AR
+es_AR.utf8
+es_BO
+es_BO.utf8
+es_CL
+es_CL.utf8
+es_CO
+es_CO.utf8
+es_EC
+es_EC.utf8
+es_ES
+es_ES@euro
+es_ES.utf8
+es_PY
+es_PY.utf8
+es_UY
+es_UY.utf8
+es_VE
+es_VE.utf8
+et_EE
+et_EE.iso885915
+et_EE.utf8
+eu_ES
+eu_ES@euro
+eu_ES.utf8
+fi_FI
+fi_FI@euro
+fi_FI.utf8
+fo_FO
+fo_FO.utf8
+fr_BE
+fr_BE@euro
+fr_BE.utf8
+fr_CA
+fr_CA.utf8
+fr_CH
+fr_CH.utf8
+fr_FR
+fr_FR@euro
+fr_FR.utf8
+fr_LU
+fr_LU@euro
+fr_LU.utf8
+gl_ES
+gl_ES@euro
+gl_ES.utf8
+hr_HR
+hr_HR.utf8
+hu_HU
+hu_HU.utf8
+id_ID
+id_ID.utf8
+is_IS
+is_IS.utf8
+it_CH
+it_CH.utf8
+it_IT
+it_IT@euro
+it_IT.utf8
+ka_GE
+ka_GE.utf8
+kk_KZ
+kk_KZ.utf8
+kl_GL
+kl_GL.utf8
+lt_LT
+lt_LT.utf8
+lv_LV
+lv_LV.utf8
+mk_MK
+mk_MK.utf8
+mn_MN
+mn_MN.utf8
+nb_NO
+nb_NO.utf8
+nl_BE
+nl_BE@euro
+nl_BE.utf8
+nl_NL
+nl_NL@euro
+nl_NL.utf8
+nn_NO
+nn_NO.utf8
+no_NO
+no_NO.utf8
+oc_FR
+oc_FR.utf8
+pl_PL
+pl_PL.utf8
+pt_BR
+pt_BR.utf8
+pt_PT
+pt_PT@euro
+pt_PT.utf8
+ro_RO
+ro_RO.utf8
+ru_RU
+ru_RU.koi8r
+ru_RU.utf8
+ru_UA
+ru_UA.utf8
+se_NO
+se_NO.utf8
+sh_YU
+sh_YU.utf8
+sk_SK
+sk_SK.utf8
+sl_SI
+sl_SI.utf8
+sq_AL
+sq_AL.utf8
+sr_CS
+sr_CS.utf8
+sv_FI
+sv_FI@euro
+sv_FI.utf8
+sv_SE
+sv_SE.iso885915
+sv_SE.utf8
+tg_TJ
+tg_TJ.utf8
+tr_TR
+tr_TR.utf8
+tt_RU.utf8
+uk_UA
+uk_UA.utf8
+vi_VN
+vi_VN.tcvn
+wa_BE
+wa_BE@euro
+wa_BE.utf8
+
